@@ -22,6 +22,8 @@
 #ifndef _GUI_H
 #define _GUI_H
 
+#include <gtk/gtk.h>
+
 /* Main window functions */
 void gui_create();
 void gui_main_refresh();
@@ -41,6 +43,7 @@ void gui_timer_refresh(unsigned int);
 void gui_tempo_show();
 void gui_tempo_hide();
 void gui_tempo_changed(GtkAdjustment *, gpointer);
+void gui_tpl_changed(GtkAdjustment *, gpointer);
 
 /* Section list window functions */
 void gui_section_show();
