@@ -199,7 +199,7 @@ void song_settempo(struct song *song, unsigned char tempo) {
 /* Converts an MMD2 module to a song */
 struct song *song_convertMMD2(struct MMD2 *mmd) {
   struct song *song;
-  int i, j, k;
+  int i, j, k, l;
   unsigned char effectconversion[256];
 
   if(mmd==NULL)
@@ -284,7 +284,7 @@ struct song *song_convertMMD2(struct MMD2 *mmd) {
       unsigned char *notes=(unsigned char *)MMD1Block+sizeof(struct MMD1Block);
       if(MMD1Block->info->pagetable)
 	block=block_alloc(MMD1Block->numtracks, MMD1Block->lines+1,
-			  MMD1Block->info->pagetable->num_pages);
+			  MMD1Block->info->pagetable->num_pages+1);
       else
 	block=block_alloc(MMD1Block->numtracks, MMD1Block->lines+1, 1);
 
@@ -308,7 +308,24 @@ struct song *song_convertMMD2(struct MMD2 *mmd) {
 	}
 
       if(MMD1Block->info->pagetable) {
-	/* FIX: implement this stuff (multiple pages) */
+	for(j=0; j<MMD1Block->info->pagetable->num_pages; j++) {
+	  for(k=0; k<(MMD1Block->lines+1); k++)
+	    for(l=0; l<MMD1Block->numtracks; l++) {
+	      block->effects[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2]=effectconversion[MMD1Block->info->pagetable->page[j][(k*MMD1Block->numtracks+l)*2]];
+	      block->effects[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2+1]=MMD1Block->info->pagetable->page[j][(k*MMD1Block->numtracks+l)*2+1];
+
+	      /* Effect values may need special handling */
+	      /* Fix volume range to 0..127 */
+	      switch(block->effects[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2]) {
+	      case EFFECT_VELOCITY:
+		if(block->effects[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2+1]==64)
+		  block->effects[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2+1]=127;
+		else
+		  block->effects[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2+1]*=2;
+		break;
+	      }
+	    }
+	}
       }
     }
     /* Instruments */
